@@ -1,8 +1,9 @@
 import struct
-from enum import Enum
 import sys
 import math
 from .encoding import encode as base_pack, decode as unpack
+from .object_update import decode_obj_update_packet
+from .enumerations import *
 
 def pack(fmt, *args):
     return base_pack(fmt, args)
@@ -10,10 +11,6 @@ def pack(fmt, *args):
 class SoftDecodeFailure(RuntimeError):
     pass
 
-class PacketProvenance(Enum):
-    server = 0x01
-    client = 0x02
-
 PACKETS = {}
 
 def packet(n):
@@ -78,14 +75,6 @@ class VersionPacket:
     def __str__(self):
         return "<VersionPacket {}.{}.{}>".format(self.major, self.minor, self.patch)
 
-class GameType(Enum):
-    siege = 0
-    single_front = 1
-    double_front = 2
-    deep_strike = 3
-    peacetime = 4
-    border_war = 5
-
 @packet(0x3de66711)
 class DifficultyPacket:
     def __init__(self, difficulty, game_type):
@@ -103,23 +92,6 @@ class DifficultyPacket:
     def __str__(self):
         return "<DifficultyPacket difficulty={} game_type={}>".format(self.difficulty, self.game_type)
 
-class Console(Enum):
-    main_screen = 0
-    helm = 1
-    weapons = 2
-    engineering = 3
-    science = 4
-    comms = 5
-    data = 6
-    observer = 7
-    captain_map = 8
-    game_master = 9
-
-class ConsoleStatus(Enum):
-    available = 0
-    yours = 1
-    unavailable = 2
-
 @packet(0x19c6e2d4)
 class ConsoleStatusPacket:
     def __init__(self, ship, consoles):
@@ -176,6 +148,83 @@ class IntelPacket:
     def __str__(self):
         return '<IntelPacket object={0} intel={1!r}>'.format(self.object, self.intel)
 
+@packet(0xd672c35f)
+class CommsIncomingPacket:
+    def __init__(self, priority, sender, message):
+        self.priority = priority
+        self.sender = sender
+        self.message = message
+
+    def encode(self):
+        return pack('Iuu', self.priority, self.sender, self.message.replace('\n', '^'))
+
+    @classmethod
+    def decode(cls, packet):
+        prio, sender, message = unpack('Iuu', packet)
+        return cls(prio, sender, message.replace('^', '\n'))
+
+    def __str__(self):
+        return '<CommsIncomingPacket priority={0} sender={1!r} message={2!r}>'.format(self.priority, self.sender, self.message)
+
+@packet(0x80803df9)
+class ObjectUpdatePacket:
+    def __init__(self, raw_data):
+        self.raw_data = raw_data
+
+    @property
+    def _records(self):
+        return decode_obj_update_packet(self.raw_data)
+
+    @property
+    def records(self):
+        try:
+            return self._records
+        except Exception:
+            return []
+
+    @classmethod
+    def decode(cls, packet):
+        if packet == b'\x00\x00\x00\x00':
+            return NoisePacket()
+        return cls(packet)
+
+    def encode(self):
+        return self.raw_data
+
+    def __str__(self):
+        try:
+            records = repr(self._records)
+            return '<ObjectUpdatePacket records={}>'.format(records)
+        except Exception as e:
+            return '<ObjectUpdatePacket data={0!r} error={1!r}>'.format(self.raw_data, e)
+
+class NoisePacket:
+    def __init__(self):
+        self.packet_id = 0x80803df9
+
+    def encode(self):
+        return b'\x00\x00\x00\x00'
+
+    def __str__(self):
+        return '<NoisePacket>'
+
+@packet(0xcc5a3e30)
+class DestroyObjectPacket:
+    def __init__(self, type, object):
+        self.type = type
+        self.object = object
+
+    def encode(self):
+        return pack('BI', self.type.value, self.object)
+
+    @classmethod
+    def decode(cls, packet):
+        type, object = unpack('BI', packet)
+        return cls(type=ObjectType(type), object=object)
+
+    def __str__(self):
+        return '<DestroyObjectPacket type={0!r} object={1!r}>'.format(self.type, self.object)
+
 @packet(0xf754c8fe)
 class GameMessagePacket:
     @classmethod
@@ -187,6 +236,8 @@ class GameMessagePacket:
             return GameStartPacket.decode(packet)
         if subtype_index == 6:
             return GameEndPacket.decode(packet)
+        if subtype_index == 9:
+            return SkyboxPacket.decode(packet)
         if subtype_index == 10:
             return PopupPacket.decode(packet)
         if subtype_index == 11:
@@ -195,6 +246,8 @@ class GameMessagePacket:
             return JumpStartPacket.decode(packet)
         if subtype_index == 13:
             return JumpEndPacket.decode(packet)
+        if subtype_index == 15:
+            return AllShipSettingsPacket.decode(packet)
         if subtype_index == 16:
             return DmxPacket.decode(packet)
         raise SoftDecodeFailure()
@@ -225,6 +278,26 @@ class GameEndPacket(GameMessagePacket):
     def __str__(self):
         return '<GameEndPacket>'
 
+class AllShipSettingsPacket(GameMessagePacket):
+    def __init__(self, ships):
+        self.ships = list(ships)
+        if len(self.ships) != 8:
+            raise ValueError('Must be 8 ships, {} given'.format(len(self.ships)))
+
+    def encode(self):
+        return pack('I[IIIu]', 15,
+                    [(ship.drive.value, ship.type.value, 1, ship.name)
+                        for ship in self.ships])
+
+    @classmethod
+    def decode(cls, packet):
+        _id, records = unpack('I[IIIu]', packet)
+        return cls(ShipSettingsRecord(DriveType(drv), ShipType(typ), name)
+                      for drv, typ, _what, name in records)
+
+    def __str__(self):
+        return '<AllShipSettingsPacket settings={0!r}>'.format(self.ships)
+
 class JumpStartPacket(GameMessagePacket):
     def encode(self):
         return b'\x0c\x00\x00\x00'
@@ -267,6 +340,21 @@ class DmxPacket(GameMessagePacket):
     def __str__(self):
         return '<DmxPacket flag={0!r} state={1!r}>'.format(self.flag, self.state)
 
+class SkyboxPacket(GameMessagePacket):
+    def __init__(self, skybox):
+        self.skybox = skybox
+
+    def encode(self):
+        return pack('II', 9, self.skybox)
+
+    @classmethod
+    def decode(cls, packet):
+        _id, skybox = unpack('II', packet)
+        return cls(skybox)
+
+    def __str__(self):
+        return '<SkyboxPacket skybox={0!r}>'.format(self.skybox)
+
 class PopupPacket(GameMessagePacket):
     def __init__(self, message):
         self.message = message
@@ -297,15 +385,6 @@ class AutonomousDamconPacket(GameMessagePacket):
     def __str__(self):
         return '<AutonomousDamconPacket autonomy={0!r}>'.format(self.autonomy)
 
-class MainView(Enum):
-    forward = 0
-    port = 1
-    starboard = 2
-    aft = 3
-    tactical = 4
-    lrs = 5
-    status = 6
-
 @packet(0x4c821d3c)
 class ShipAction1Packet:
     @classmethod
